!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Copyright (C) 2010 Timothy Brown
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 3 of the License, or (at
! your option) any later version.
!
! This program is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
! MA 02110-1301, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****im* dom_ti
!
!  NAME
!    dom_ti
!
!  DESCRIPTION
!    The module dom_ti provides subroutines to access
!    time independent variables from a binary wrf file.
!
!****
module dom_ti

    implicit none

    private

    public :: print_char
    public :: print_int
    public :: print_int_array
    public :: print_real
    public :: print_real_array

    contains

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****im* dom_ti/dom_ti_print
!
!  NAME
!    dom_ti_print
!
!  DESCRIPTION
!    The subroutine dom_ti_print will retreive and print to stdout
!    a time independent variable from a binary wrf file.
!
!  INPUT
!    * ifd    - The binary WRF open file descriptor to read from.
!    * vname  - The variable name to read from the file.
!    * vtype  - Integer flag indicating the type of the variable.
!              0 => character
!              1 => integer
!              2 => real
!              3 => double precision
!
!  RETURNS
!****
    subroutine dom_ti_print(ifd, vname, vtype, vlen)

        use module_ext_internal
        implicit none

        integer, intent(in)               :: ifd   ! File descriptor
        character(len=*), intent(in)      :: vname ! Variable to read from file
        integer, intent(in)               :: vtype ! Variable type
        integer                           :: vlen  ! Length of the array to read
        character(len=255)                :: c_v   ! Value (if char)
        integer, dimension(vlen)          :: i_v   ! Value (if int)
        real, dimension(vlen)             :: r_v   ! Value (if real)
        double precision, dimension(vlen) :: d_v   ! Value (if double)
        integer                           :: n     ! Number of digits read
        integer                           :: ierr  ! Error status

        c_v = ' '
        i_v = 0
        r_v = 0.0
        d_v = 0.0

        select case(vtype)
        case (0)
            call ext_int_get_dom_ti_char(ifd, vname, c_v, ierr)
        case (1)
            call ext_int_get_dom_ti_integer(ifd, vname, i_v, vlen, n, ierr)
        case (2)
            call ext_int_get_dom_ti_real(ifd, vname, r_v, vlen, n, ierr)
        case (3)
            call ext_int_get_dom_ti_real(ifd, vname, d_v, vlen, n, ierr)
        end select

        if (ierr /= 0) then
            write(0,*) 'Unable to retreive ', trim(vname)
            return
        endif

        write(6, '(A,t32)', advance="no") trim(vname)

        select case(vtype)
        case (0)
            write(6, *) trim(c_v)
        case (1)
            write(6, *) i_v
        case (2)
            write(6, *) r_v
        case (3)
            write(6, *) d_v
        end select

    end subroutine dom_ti_print

!****im* dom_ti/print_char
!
!  NAME
!    print_char
!
!  DESCRIPTION
!    The subroutine print_char will retreive and print to stdout
!    a time independent character array variable from a binary wrf file.
!
!  INPUT
!    * ifd   - The binary WRF open file descriptor to read from
!    * name  - The variable name to read from the file
!
!  RETURNS
!****
    subroutine print_char(ifd, name)
        implicit none
        integer, intent(in)            :: ifd     ! File descriptor
        character(len=*), intent(in)   :: name    ! Variable name

        call dom_ti_print(ifd, name, 0, 0)

        return
    end subroutine print_char

!****im* dom_ti/print_int
!
!  NAME
!    print_int
!
!  DESCRIPTION
!    The subroutine print_int will retreive and print to stdout
!    a time independent integer variable from a binary wrf file.
!
!  INPUT
!    * ifd   - The binary WRF open file descriptor to read from
!    * name  - The variable name to read from the file
!
!  RETURNS
!****
    subroutine print_int(ifd, name)
        implicit none
        integer, intent(in)            :: ifd     ! File descriptor
        character(len=*), intent(in)   :: name    ! Variable name

        call dom_ti_print(ifd, name, 1, 1)

        return
    end subroutine print_int

!****im* dom_ti/print_int_array
!
!  NAME
!    print_int_array
!
!  DESCRIPTION
!    The subroutine print_int_array will retreive and print to stdout
!    a time independent integer array variable from a binary wrf file.
!
!  INPUT
!    * ifd   - The binary WRF open file descriptor to read from
!    * name  - The variable name to read from the file
!    * len   - The length of the array (number of elements)
!
!  RETURNS
!****
    subroutine print_int_array(ifd, name, len)
        implicit none
        integer, intent(in)            :: ifd     ! File descriptor
        character(len=*), intent(in)   :: name    ! Variable name
        integer, intent(in)            :: len     ! Variable length

        call dom_ti_print(ifd, name, 1, len)

        return
    end subroutine print_int_array

!****im* dom_ti/print_real
!
!  NAME
!    print_real
!
!  DESCRIPTION
!    The subroutine print_real will retreive and print to stdout
!    a time independent real variable from a binary wrf file.
!
!  INPUT
!    * ifd   - The binary WRF open file descriptor to read from
!    * name  - The variable name to read from the file
!
!  RETURNS
!****
    subroutine print_real(ifd, name)
        implicit none
        integer, intent(in)            :: ifd     ! File descriptor
        character(len=*), intent(in)   :: name    ! Variable name

        call dom_ti_print(ifd, name, 2, 1)

        return
    end subroutine print_real

!****im* dom_ti/print_real_array
!
!  NAME
!    print_real_array
!
!  DESCRIPTION
!    The subroutine print_real_array will retreive and print to stdout
!    a time independent real array variable from a binary wrf file.
!
!  INPUT
!    * ifd   - The binary WRF open file descriptor to read from
!    * name  - The variable name to read from the file
!    * len   - The length of the array (number of elements)
!
!  RETURNS
!****
    subroutine print_real_array(ifd, name, len)
        implicit none
        integer, intent(in)            :: ifd     ! File descriptor
        character(len=*), intent(in)   :: name    ! Variable name
        integer, intent(in)            :: len     ! Variable length

        call dom_ti_print(ifd, name, 2, len)

        return
    end subroutine print_real_array

end module dom_ti
