!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!
! Copyright (C) 2010 Timothy Brown
!
! This program is free software; you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation; either version 3 of the License, or (at
! your option) any later version.
!
! This program is distributed in the hope that it will be useful, but
! WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! General Public License for more details.
!
! You should have received a copy of the GNU General Public License
! along with this program; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
! MA 02110-1301, USA.
!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!****im* meta_type
!
!  NAME
!    meta_type
!
!  DESCRIPTION
!    The module meta_type provides meta_t, a meta data type for the
!    geogrid time independent data attributes.
!
!****
module meta_type
    implicit none
    private
    public :: meta_t
    public :: meta_t_init
    public :: meta_t_destroy
    public :: meta_t_add
    public :: meta_t_reset
    public :: meta_t_update
!    public :: meta_t_delete
    public :: meta_t_retrieve
    public :: meta_t_dump
    public :: meta_t_size
    public :: meta_t_max

    type :: meta_t
        integer            :: sr_x
        integer            :: sr_y
        character(len=128) :: name
        character(len=128) :: units
        character(len=128) :: description
        character(len=128) :: stagger
    end type meta_t

    type(meta_t), allocatable, dimension(:) :: list  ! Internal Array/list
    integer                                 :: l_max ! Maximum size of list
    integer                                 :: l_n   ! Current list size
    integer, parameter                      :: LIST_MAX = 50 ! Default max
                                               ! size of the array/list

    contains

    subroutine meta_t_init(n)
        implicit none
        integer, intent(in), optional :: n ! size of list to allocate

        if (present(n)) then
            l_max = n
        else
            l_max = LIST_MAX
        endif

        if (.not. allocated(list)) then
            allocate(list(l_max))
            l_n = 1
        endif

    end subroutine meta_t_init

    subroutine meta_t_destroy
        implicit none

        if (allocated(list)) then
            deallocate(list)
            l_max = 0
            l_n = 0
        endif

    end subroutine meta_t_destroy

    subroutine meta_t_add(element)
        implicit none

        type(meta_t), intent(in)                :: element ! Element to add
        type(meta_t), allocatable, dimension(:) :: tmp     ! Temporary list
        integer                                 :: n_n     ! Temporary size
        integer                                 :: n_max   ! New max

        if (allocated(list)) then
            if (l_n .le. l_max) then
                list(l_n) = element
                l_n = l_n + 1
            else
                n_max = l_max * 2
                n_n = l_n
                allocate(tmp(l_max))
                tmp = list
                call meta_t_destroy
                call meta_t_init(n_max)
                list = tmp
                l_n = n_n
                deallocate(tmp)
            endif
        endif

    end subroutine meta_t_add

    subroutine meta_t_reset(element)
        implicit none

        type(meta_t), intent(inout)             :: element ! Element to reset

        element%name = ' '
        element%description = ' '
        element%units = ' '
        element%stagger = ' '
        element%sr_x = 0
        element%sr_y = 0

    end subroutine meta_t_reset

    subroutine meta_t_update(i, element)
        implicit none

        integer, intent(in)                     :: i ! Element number to update
        type(meta_t), intent(in)                :: element ! Element to update

        if (allocated(list)) then
            if (len(trim(element%name)) .gt. 0) then
                list(i)%name = element%name
            endif
            if (len(trim(element%description)) .gt. 0) then
                list(i)%description = element%description
            endif
            if (len(trim(element%units)) .gt. 0) then
                list(i)%units = element%units
            endif
            if (len(trim(element%stagger)) .gt. 0) then
                list(i)%stagger = element%stagger
            endif
            if (element%sr_x .ne. 0) then
                list(i)%sr_x = element%sr_x
            endif
            if (element%sr_y .ne. 0) then
                list(i)%sr_y = element%sr_y
            endif
        endif

    end subroutine meta_t_update

!    subroutine meta_t_delete
!        implicit none
!    end subroutine meta_t_delete

    subroutine meta_t_retrieve(name, element, ierr)
        implicit none

        character(len=*), intent(in) :: name
        type(meta_t), intent(out)    :: element
        integer                      :: ierr
        integer                      :: i

        ierr = 1
        do i=1,l_n
          if (list(i)%name .eq. name) then
              element = list(i)
              ierr = 0
              exit
          endif
        end do

    end subroutine meta_t_retrieve

    subroutine meta_t_dump()
        implicit none

        integer                      :: i

        do i=1,l_n
          write(6,'(4(A,A,/),2(A,I,/))')                              &
                'Name:        ', trim(list(i)%name),                  &
                'Description: ', trim(list(i)%description),           &
                'Units:       ', trim(list(i)%units),                 &
                'Stagger:     ', trim(list(i)%stagger),               &
                'Sr_x:        ', list(i)%sr_x,                        &
                'Sr_y:        ', list(i)%sr_y
        end do

    end subroutine meta_t_dump

    subroutine meta_t_size(n)
        implicit none
        integer, intent(out)         :: n

        n = l_n

    end subroutine meta_t_size

    subroutine meta_t_max(n)
        implicit none
        integer, intent(out)         :: n

        n = l_max

    end subroutine meta_t_max

end module meta_type
